package lab10_solutions;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.Comparator;
import java.util.Scanner;


public class TestExtendedBinaryTree {
	private static Scanner in = new Scanner(System.in);

	  
	  /**
	  * We have a txt file and we load it into a tree
	  * structure. A node of the tree includes name and phone
	  * number of a person.
	  * @param args
	  */
	  public static void main(String[] args)
	    {
		  Comparator<PhoneRec2> c1 = new Comparator<PhoneRec2>() {

			@Override
			public int compare(PhoneRec2 o1, PhoneRec2 o2) {
				return o1.getName().compareToIgnoreCase(o2.getName());
			}
			  
		  };
		  
		  Comparator<PhoneRec2> c2 = new Comparator<PhoneRec2>() {

			@Override
			public int compare(PhoneRec2 o1, PhoneRec2 o2) {
				return o1.getSurname().compareToIgnoreCase(o2.getSurname());
			}
			  
		  };
		  
	      ExtendedBinaryTree<PhoneRec2> nameOrdered = new myExtendedBinaryTree<PhoneRec2>(c1);
	      ExtendedBinaryTree<PhoneRec2> surnameOrdered  = new myExtendedBinaryTree<PhoneRec2>(c2);

	      System.out.println("\nLoading tree1 from a text file ...");
	      loadTree(nameOrdered);
	      System.out.println("\nLoading tree2 from a text file ...");
	      loadTree(surnameOrdered );
	      
	      System.out.println("\nPrint Tree 1 - (nameOrdered)\n" + nameOrdered.SortedRepresentation() );  
	      System.out.println("\nPrint Tree 2 - (surnameOrdered)\n" + surnameOrdered.SortedRepresentation() );  
	      
	      	/** OUTPUT:
Loading tree1 from a text file ...

Loading tree2 from a text file ...

Print Tree 1 - (nameOrdered)
aa joh: 243321543,bb jan: 94939121,cc jodi: 6727777,dd to: 6794321,ee barbi: 6086666,ff ke: 6086666,gg sta: 6724567,kk ann: 53352532,pp mari: 3256312,zz giorgo: 3242344,

Print Tree 2 - (surnameOrdered)
kk ann: 53352532,ee barbi: 6086666,zz giorgo: 3242344,bb jan: 94939121,cc jodi: 6727777,aa joh: 243321543,ff ke: 6086666,pp mari: 3256312,gg sta: 6724567,dd to: 6794321,

	      	 */
	    }

	    /**
	      Load phone information from a text file into a binary search tree.
	      @param bst the binary search tree
	    */
	    public static void loadTree(ExtendedBinaryTree<PhoneRec2> bst)
	    {
	      try
	      {
	        FileReader reader = new FileReader("phoneinfo.txt");
	        BufferedReader inFile = new BufferedReader(reader);

	        /* Each iteration of this loop reads phone information for
	           one person from inFile into a PhoneRecord object and
	           inserts that object into the binary search tree bst. */

	        while (true)
	        {
	          String inputLine = inFile.readLine();

	          if (inputLine == null) // end of file
	            break;

	          PhoneRec2 rec = parsePhoneInfo(inputLine);
	          
	          if (! bst.insert(rec) ) {
	          	System.out.println("Error inserting record into tree "+rec);
	          }
	        }

	        reader.close();
	      }

	      catch(IOException e)
	      {
	        System.out.println("Input/output error " + e);
	        System.out.println("Probable cause: incorrect file format.\n");
	      }
	    }

	    /**
	      Creates a PhoneRecord object from phone information in a string.
	      @param line the string from which the phone information is extracted
	      @return PhoneRecord object containing the phone information
	    */
	    public static PhoneRec2 parsePhoneInfo(String line)
	    {
	      int i = 0;
	      while (i < line.length() && ! (line.charAt(i) == ' '))
	        i++;

	      String nameInfo = line.substring(0,i);
	      int j = i + 1;
	      while (j < line.length() && ! (line.charAt(j) == ' '))
		        j++;

		  String surnameInfo = line.substring(i+1,j-1);
		     
	      int numInfo = Integer.parseInt(line.substring(j+1));
	      PhoneRec2 rec = new PhoneRec2(nameInfo, surnameInfo, numInfo);
	    //  System.out.println(rec);
	      return rec;
	    }



}
